/***************************************************************************
                          appwindow.cpp  -  description
                             -------------------
    begin                : Sun Jul 16 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : joris@struyve.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: appwindow.cpp,v 1.19 2002/12/21 10:43:58 js Exp $
 *
 */

#include "k8ktest.h"

bool interface_ok;

ApplicationWindow::ApplicationWindow(QString conn)
    : QMainWindow( 0, "MainWindow" )
{
	int i=0;
	int posw=45;
	int buttonw=80;
	int buttonh=25;

	/* use mutexes in libk8000 */
	SetMutexlock(1);

	/* create a timer to scan the k8000 card */
	timer = new QTimer( this );

	/* create the settins class */
	settings = new Settings();

	/* make a menu */
	this->menu = new MenuBar(this,"menu");

	/* create the toolbar */
	toolbar = new ToolBar(this,"toolbar");
	this->addToolBar(toolbar);

	/* create the statusbar */
	statusbar = new StatusBar(this,"statusbar");

	/* Create the optionsdialog */
	this->optionsdialog = new OptionsDialog(this,"optionsdialog");

	/* Create the mainframe and set as central widget */
	mainframe = new QFrame(this,"mainframe");
	this->setCentralWidget(mainframe);

	/* Create the analogoframe to surround the analog outputs */
	analogoframe = new QFrame(mainframe,"analogoframe");
	
	/* Create the analogoframe to surround the analog outputs */
	analogiframe = new QFrame(mainframe,"analogiframe");
	
	/* Create the analogoframe to surround the analog outputs */
	digitalioframe = new QFrame(mainframe,"digitalioframe");

	/* Create the buttonframe to surround the analog outputs */
	buttonframe = new QFrame(mainframe,"buttonframe");
	buttonframe->setGeometry(640,165,120,200);

	/*Create the buttons*/
	alli = new QPushButton(buttonframe,"alli");
	alli->setText("All input");
	alli->setGeometry(0,0,buttonw,buttonh);
	
	allo = new QPushButton(buttonframe,"allo");
	allo->setText("All output");
	allo->setGeometry(0,30,buttonw,buttonh);
	
	set_max = new QPushButton(buttonframe,"set_max");
	set_max->setText("All up");
	set_max->setGeometry(0,60,buttonw,buttonh);
	
	set_min = new QPushButton(buttonframe,"set_min");
	set_min->setText("All down");
	set_min->setGeometry(0,90,buttonw,buttonh);
	
	quit = new QPushButton(buttonframe,"quit");
	quit->setText("Quit");
	quit->setGeometry(0,120,buttonw,buttonh);

	/* Create a digitalios  */
	int jump=0;
	for (i=0;i<MaxIOchannel;i++)
	{
		if (!(i%16)&&i!=0)
			jump=jump+16;
			
		digitalio[i] = new DigitalIo(0,i+1,QString("IO")+tr("%1").arg(i+1),digitalioframe,"io");
		digitalio[i]->SetGeometry((i-jump)*posw,0,posw,100,18);
		//digitalio[i]->SetIoType(FALSE);
	}
	digitalioframe->setGeometry(5,5,(i-jump)*posw,100);

	/* Create a analogoutputs */
	jump=0;
	for (i=0;i<MaxDACchannel;i++)
	{
		if (!(i%8)&&i!=0)
			jump=jump+8;

		analogDACoutput[i] = new AnalogOutput(0,63,1,10,0,i+1,QString("DAC"),
											  QString("DAC")+tr("%1").arg(i+1),analogoframe,"io");
		analogDACoutput[i]->SetGeometry((i-jump)*posw,0,posw,200,18);
	}

	for (i=0;i<MaxDAchannel;i++)
	{
		analogDAoutput[i] = new AnalogOutput(0,255,1,10,0,i+1,QString("DA"),
											 QString("DA")+tr("%1").arg(i+1),analogoframe,"io");
		analogDAoutput[i]->SetGeometry(posw*8,0,posw,200,18);
	}
	analogoframe->setGeometry(5,digitalioframe->height()+10,9*posw,200);

	/* Create a analoginputs */
	jump=0;
	for (i=0;i<MaxADchannel;i++)
	{
		if (!(i%4)&&i!=0)
			jump=jump+4;
			
		analoginput[i] = new AnalogInput(0,255,1,10,0,i+1,QString("AD")+tr("%1").arg(i+1),analogiframe,"io");
		analoginput[i]->SetGeometry((i-jump)*posw,0,posw,200,18);
	}
	analogiframe->setGeometry(5+analogoframe->width()+posw,digitalioframe->height()+10,(i-jump)*posw,200);

	/* Connect the slots and signals */
	QObject::connect( timer,       SIGNAL(timeout()), this, SLOT(Scan()) );
	QObject::connect( quit,        SIGNAL(clicked()), this, SLOT(Exit()) );
	QObject::connect( allo,        SIGNAL(clicked()), this, SLOT(AllOutput()) );
	QObject::connect( alli,        SIGNAL(clicked()), this, SLOT(AllIntput()) );
	QObject::connect( set_min,     SIGNAL(clicked()), this, SLOT(SetMinimum()) );
	QObject::connect( set_max,     SIGNAL(clicked()), this, SLOT(SetMaximum()) );

	/* size the main window */
	this->resize(730,400);
	this->RestoreSettings();

	/* start the scan timer */
	timer->start( settings->refreshrate, FALSE );

}

ApplicationWindow::~ApplicationWindow()
{}

void ApplicationWindow::Scan()
{
	int i;

	if (interface_ok)
	{
	    statusbar->SetStatusString(QString("Refreshing..."));
		this->statusbar->Blink();	
		/* refresh all io */
		for (i=0;i<MaxIOchannel;i++)
			digitalio[i]->Refresh();
		for (i=0;i<MaxDACchannel;i++)
			analogDACoutput[i]->Refresh();
		for (i=0;i<MaxDAchannel;i++)
			analogDAoutput[i]->Refresh();
		for (i=0;i<MaxADchannel;i++)
			analoginput[i]->Refresh();
		statusbar->SetStatusString(QString("Ready"));
	}
}

void ApplicationWindow::RestoreSettings()
{
	int i;
	/* call restore to the settings */
	settings->RestoreSettings();
	this->SetNewOptions();

	for (i=0;i<MaxDACchannel;i++)
		analogDACoutput[i]->SetValue(settings->aoDACstate[i]);

	for (i=0;i<MaxDAchannel;i++)
		analogDAoutput[i]->SetValue(settings->aoDAstate[i]);

	for (i=0;i<MaxIOchannel;i++)
	{
		digitalio[i]->SetIoType(settings->iotype[i]);
		digitalio[i]->SetValue(settings->iostate[i]);	
	}
}

void ApplicationWindow::SaveSettings()
{
	/* call save to the settings object*/
	int i;
	for (i=0;i<MaxDACchannel;i++)
		settings->aoDACstate[i] = analogDACoutput[i]->GetValue();
	for (i=0;i<MaxDAchannel;i++)
		settings->aoDAstate[i] = analogDAoutput[i]->GetValue();
	for (i=0;i<MaxIOchannel;i++)
	{
		settings->iotype[i]  = digitalio[i]->GetIoType();
		settings->iostate[i] = digitalio[i]->GetValue();	
	}
	settings->saveonexit = menu->GetSaveonexitState();
	settings->SaveSettings();
}


void ApplicationWindow::AllOutput()
{
	int i;
	for (i=0;i<MaxIOchannel;i++)
		digitalio[i]->SetIoType(FALSE);	
}

void ApplicationWindow::AllIntput()
{
	int i;
	for (i=0;i<MaxIOchannel;i++)
		digitalio[i]->SetIoType(TRUE);	
}

void ApplicationWindow::SetMinimum()
{
	int i;
	for (i=0;i<MaxDACchannel;i++)
		analogDACoutput[i]->SetValue(0);
	for (i=0;i<MaxDAchannel;i++)
		analogDAoutput[i]->SetValue(0);
	for (i=0;i<MaxIOchannel;i++)
		digitalio[i]->SetValue(FALSE);	

}

void ApplicationWindow::SetMaximum()
{
	int i;
	for (i=0;i<MaxDACchannel;i++)
		analogDACoutput[i]->SetValue(63);
	for (i=0;i<MaxDAchannel;i++)
		analogDAoutput[i]->SetValue(255);
	for (i=0;i<MaxIOchannel;i++)
		digitalio[i]->SetValue(TRUE);	
}

	
void ApplicationWindow::ShowAbout()
{		
	AboutDialog about(this,"About");
	about.exec();
}

void ApplicationWindow::ShowOptionDialog()
{
	OptionsDialog od(this,"Login");
	if ( od.exec() )
	{
		settings->refreshrate = od.GetRefreshrate();
		settings->i2cbusdelay = od.GetI2cBusDelay();
		settings->lptport     = od.GetLptport();
	}
	this->SetNewOptions();
}

void ApplicationWindow::ShowI2cDialog()
{
	I2cDialog d(this,"Login");
	d.exec();
}

void ApplicationWindow::ShowFontDialog()
{
	bool ok;

	QFont fnt = QFontDialog::getFont( &ok,this->font(), this );
    if ( ok )
	{
        QApplication::setFont( fnt ,TRUE);
		settings->font = fnt.toString();
	}
}

void ApplicationWindow::ShowCard(int cardnumber)
{
	int i;
	
	toolbar->SetCard(cardnumber);
	settings->currentcard = cardnumber;
	/* show the selected card's io's */
	for (i=0;i<MaxIOchannel;i++)
	{
		if(i>=(cardnumber*16) && i<(16+(cardnumber*16)) )
			digitalio[i]->show();
		else
			digitalio[i]->hide();
	}
	for (i=0;i<MaxDACchannel;i++)
	{
		if(i>=(cardnumber*8) && i<(8+(cardnumber*8)) )
			analogDACoutput[i]->show();
		else
			analogDACoutput[i]->hide();
	}
	for (i=0;i<MaxDAchannel;i++)
	{
		if(i>=cardnumber && i<(1+cardnumber) )
			analogDAoutput[i]->show();
		else
			analogDAoutput[i]->hide();

	}
	for (i=0;i<MaxADchannel;i++)
	{
		if(i>=(cardnumber*4) && i<(4+(cardnumber*4)) )
			analoginput[i]->show();
		else
			analoginput[i]->hide();
	}
}

void ApplicationWindow::ShowChain(int chainnumber)
{

	settings->currentchain = chainnumber;
	toolbar->SetChain(chainnumber);
	SelectChain(chainnumber);
}

void ApplicationWindow::SetNewOptions()
{
	QFont fnt;
	if (!settings->font.isEmpty())
		fnt.fromString(settings->font);
	QApplication::setFont( fnt ,TRUE);

	ShowChain(settings->currentchain);
	ShowCard(settings->currentcard);

	menu->SetSaveonexitState(settings->saveonexit);

	if(SelectI2CprinterPort(settings->lptport))
	{
		QMessageBox::message("Error","Error selecting printer port or device.");
		statusbar->SetStatusString(QString("Interface error"));
		interface_ok = false;
		//exit(1);
	}
	else
		interface_ok = true;

	timer->changeInterval( settings->refreshrate );
	I2CbusDelay=settings->i2cbusdelay;
}

void ApplicationWindow::Exit()
{
	if (menu->GetSaveonexitState())
		SaveSettings();
	qApp->quit();
}

void ApplicationWindow::NullSlot()
{/* This method does nothing (as it's name sugests) */}

