/***************************************************************************
                          digitalio.cpp  -  description
                             -------------------
    begin                : Mon Jul 17 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : joris@struyve.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: digitalio.cpp,v 1.11 2002/12/21 10:43:58 js Exp $
 *
 */

#include "k8ktest.h"

DigitalIo::DigitalIo(bool value,int io,QString label,QWidget *parent,char *name)
	:QFrame(parent,name)
{
  	this->par = parent;
	this->setFrameStyle(QFrame::Box|QFrame::Raised);
	val = new QToolButton(this,"val" );
	val->setToggleButton(TRUE);
	val->setForegroundColor(QColor(255,0,0));
	
	cardlabel = new QLabel(this,"card");
	cardlabel->setAlignment(Qt::AlignHCenter);
	cardlabel->setText(label);

	io_config = new QButtonGroup(this,"group");
	i_type = new QRadioButton("I",io_config,"i");
	o_type = new QRadioButton("O",io_config,"o");
	
	connect( val, SIGNAL(clicked() ), SLOT(ChangeValue()) );
	connect( i_type, SIGNAL(toggled(bool) ), SLOT(SetIoType(bool)) );
	
	/* set the io_channel */
	this->io_channel = io;

}

DigitalIo::~DigitalIo()
{}

void DigitalIo::ChangeValue()
{
  	if (interface_ok)
	{
		/* emit the change signal */
		emit this->ValueChanged(val->isDown());
	
		/* read the current value to the K8000 card if input*/
		/* write the current value to the K8000 card if output */
		if(o_type->isChecked())
		{
			if(val->isOn())
					SetIOchannel(io_channel);
			else
				ClearIOchannel(io_channel);
		}

		/* readback the current value to the K8000 card */
		ReadIOchannel(io_channel);
		if(IO[io_channel])
			val->setOn(TRUE);
		else
			val->setOn(FALSE);
	}
}

void DigitalIo::Refresh()
{
  	if (interface_ok)
	{
		/* read the current value to the K8000 card */
		ReadIOchannel(io_channel);
		//qDebug("IO %d = %d",io_channel,IO_matrix[activechainnumber][io_channel]);
		if(IO_matrix[activechainnumber][io_channel])
			val->setOn(TRUE);
		else
			val->setOn(FALSE);
	}
}

bool DigitalIo::GetValue()
{ return val->isOn(); }

bool DigitalIo::GetIoType()
{ return i_type->isChecked(); }

void DigitalIo::SetValue(bool value)
{
  	if (interface_ok)
	{
		/*return val->value();*/
		if(value)
			SetIOchannel(io_channel);
		else
			ClearIOchannel(io_channel);
	}
}

void DigitalIo::SetIoType(bool type)
{
  	if (interface_ok)
	{
		if (type)
		{
			i_type->setChecked(TRUE);
			ConfigIOchannelAsInput(io_channel);
		}
		else
		{
			o_type->setChecked(TRUE);
			ConfigIOchannelAsOutput(io_channel);
		}
	}
}

void DigitalIo::SetGeometry(int x,int y,int w,int h,int s)
{
	int borderspace = 3;
	int interspace = 5;
	int texth = 13;
	int chkw = 32;
	int chkh = 20;
	
	this->setGeometry(x,y,w,h);

	cardlabel->setGeometry(borderspace,borderspace,w-2*borderspace,texth);
	val->setGeometry((w/2)-(s/2),texth+interspace+borderspace,s,
					h-(3*chkh)-(4*borderspace)-(2*interspace));
	
	io_config->setGeometry(borderspace,h-(4*borderspace)-(2*chkh),w-2*borderspace,(chkh*2)+(3*borderspace));
	i_type->setGeometry(borderspace,borderspace,chkw,chkh);
	o_type->setGeometry(borderspace,(2*borderspace)+chkh,chkw,chkh);
	
}
