/***************************************************************************
                          I2cDialog.cpp  -  description
                             -------------------
    begin                : Sun Aug 11 2002
    copyright            : (C) 2002 by Joris Struyve
    email                : joris@struyve.be
 ***************************************************************************/
 
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 /* CVS information :
 *
 * $Id: i2cdialog.cpp,v 1.9 2002/12/21 10:43:58 js Exp $
 *
 */

#include "k8ktest.h"

I2cDialog::I2cDialog(ApplicationWindow* parent,char *label)
		: QDialog ( (QWidget*)parent, label, TRUE )
{
	int i;

	this->setCaption ( "IC send/receive" ) ;
	par = parent;
	
	/* bottom buttons */
	send  = new QPushButton( "Send", this );
	read  = new QPushButton( "Read", this );
	close = new QPushButton( "Close", this );
	send->setGeometry (60 ,300,100,25);
	read->setGeometry (165,300,100,25);
	close->setGeometry(270,300,100,25);

	/* general params */
	chain     = new QComboBox(FALSE,this,"chain");
	address   = new QComboBox(FALSE,this,"address");
	l_chain   = new QLabel(QString("Chain"),this,"l_chain");
	l_address = new QLabel(QString("IC address"),this,"l_address");
	chain->setGeometry    (150,20,70 ,20);
	address->setGeometry  (150,45,70 ,20);
	l_chain->setGeometry  ( 20,20,120,20);
	l_address->setGeometry( 20,45,120,20);


	/* send frame */
	send_data    = new QGroupBox(QString("Data to send"),this,"send_data");

	dataformat   = new QButtonGroup(send_data);
	dataformat->setFrameStyle(QFrame::NoFrame);
	hex_data     = new QRadioButton(QString("Hexadecimal Data"),dataformat,"h");
	char_data    = new QRadioButton(QString("Character data"),dataformat,"c");
	data         = new QLineEdit(send_data,"Data");
	send_data->setGeometry ( 20,70,400,95);
	dataformat->setGeometry( 20,20,320,30);
	hex_data->setGeometry  (  5, 5,140,20);
	char_data->setGeometry (150, 5,140,20);
	data->setGeometry      (20 ,55,360,20);

	/* receive frame */
	receive_data = new QGroupBox(QString("Data received"),this,"send_data");

	length              = new QLineEdit(receive_data,"length");
	received_hex_data   = new QLineEdit(receive_data,"received_hex_data");
	received_bin_data   = new QLineEdit(receive_data,"received_bin_data");
	l_received_hex_data = new QLabel(QString("Hexadecimal"),receive_data,"l_received_hex_data");
	l_received_bin_data = new QLabel(QString("Binary"),receive_data,"l_received_bin_data");
	l_length            = new QLabel(QString("Data length (bytes)"),receive_data,"l_length");
	receive_data->setGeometry        ( 20 ,175,400,100);
	l_length->setGeometry            ( 20 , 20,120,20);
	l_received_hex_data->setGeometry ( 20 , 45, 80,20);
	l_received_bin_data->setGeometry ( 20 , 70, 80,20);
	length->setGeometry              ( 150, 20,70 ,20);
	received_hex_data->setGeometry   ( 105, 45,280,20);
	received_bin_data->setGeometry   ( 105, 70,280,20);
    length->setValidator(new  QIntValidator(0,100,this));

	close->setDefault(TRUE);

	/* fill chain combobox */
	chain->insertItem("1");
	chain->insertItem("2");
	chain->insertItem("3");
	chain->insertItem("4");
	chain->insertItem("5");

	/* disable chain combobox if interface is /dev/velleman */
	if(parent->settings->currentchain==-1)
		chain->setDisabled(TRUE);

	/* fill the address list */
	for(i=0;i<127;i++)
		address->insertItem(QString("0x")+tr("%1").arg(i));

	/* character data is the default */
	hex_data->setChecked(TRUE);

	connect( send , SIGNAL(clicked()), SLOT(SendData()) );
	connect( read , SIGNAL(clicked()), SLOT(ReadData()) );
	connect( close, SIGNAL(clicked()), SLOT(accept  ()) );
}

I2cDialog::~I2cDialog()
{
	emit this->Closed();
}

void I2cDialog::resizeEvent(QResizeEvent * r_event)
{
	QDialog::resizeEvent(r_event);
	this->setMaximumSize(this->size());
	this->setMinimumSize(this->size());
}

void I2cDialog::SendData()
{
	byte *adr;
	byte *byte_data;
	int len;

  	if (interface_ok)
	{
		if(hex_data->isChecked())
		{
			HexToByte(data->text().upper().latin1(),&byte_data);
			len = data->text().length()/2;
		}
		else
		{
			byte_data = (byte*)malloc(sizeof(byte)*data->text().length());
			memcpy(byte_data,data->text().latin1(),data->text().length());
			len = data->text().length();
		}

		HexToByte(address->currentText().right(2).latin1(),&adr);
		SelectChain(chain->currentItem());
		I2cSendData(*adr,byte_data,len);
		SelectChain(par->settings->currentchain);

		free(byte_data);
		free(adr);
	}
	else
		QMessageBox::message("Error","Device status not ok.");
}

void I2cDialog::ReadData()
{
	byte *byte_data;
	byte *adr;
	char *result;

  	if (interface_ok)
	{
		HexToByte( address->currentText().right(2).latin1(),&adr );
		byte_data = (byte*)malloc(sizeof(byte)*length->text().toInt()*3);
		SelectChain(chain->currentItem());
		I2cReadData( *adr,byte_data,length->text().toInt() );
		SelectChain(par->settings->currentchain);
		ByteToHex(byte_data,&result,length->text().toInt());
		received_hex_data->setText( QString(result) );
		free(result);
		ByteToBin(byte_data,&result,length->text().toInt());
		received_bin_data->setText( QString(result) );

		free(result);
		free(byte_data);
		free(adr);
	}
	else
		QMessageBox::message("Error","Device status not ok.");

}
