/***************************************************************************
                          settings.cpp  -  description
                             -------------------
    begin                : Sun Jul 23 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : joris@struyve.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: settings.cpp,v 1.9 2002/08/18 18:24:26 js Exp $
 *
 */

#include "k8ktest.h"

Settings::Settings()
{
	int i;
	QString tmp;
	
	/* fill the options with defaults */
	for(i=0;i<MaxIOchannel;i++)
		iotype[i] = iostate[i] = FALSE;

	for(i=0;i<MaxDACchannel;i++)
		aoDACstate[i] = 0;

	for(i=0;i<MaxDAchannel;i++)
		aoDAstate[i] = 0;
	
	refreshrate = 500;
	lptport = 1;
	currentchain = 0;
	currentcard = 0;
	i2cbusdelay = 100;
	font=QString("");
}

Settings::~Settings()
{}
	
void Settings::SaveSettings()
{
	/* write the settings to a file */
	int i;
	QString tmp;

	/* look for a config file */
	QString configfilename;
	configfilename = QString(getenv("HOME")) + QString("/.k8ktest");

	configfile = new QFile(configfilename);
	if ( configfile->open(IO_WriteOnly) )
	{
		tmp.sprintf("#k8ktest configuration, (wil get overwritten every time)\n\n");
		configfile->writeBlock ((const char *)tmp,tmp.length());
		/* write general options */
		tmp.sprintf("\n#General options\n");
		configfile->writeBlock ((const char *)tmp,tmp.length());
		
		tmp.sprintf("refreshrate=%d\ni2cbusdelay=%d\nlptport=%d\n",refreshrate,i2cbusdelay,lptport);
		configfile->writeBlock ((const char *)tmp,tmp.length());

		tmp.sprintf("chain=%d\n",currentchain);
		configfile->writeBlock ((const char *)tmp,tmp.length());

		tmp.sprintf("card=%d\n",currentcard);
		configfile->writeBlock ((const char *)tmp,tmp.length());

		tmp.sprintf("saveonexit=%d\n",saveonexit);
		configfile->writeBlock ((const char *)tmp,tmp.length());

		tmp = ("font=" + font + "\n");
		configfile->writeBlock ((const char *)tmp,tmp.length());
		
		tmp.sprintf("\n#Analog output states\n");
		configfile->writeBlock ((const char *)tmp,tmp.length());
		/* write ao config */
		for (i=0;i<MaxDACchannel;i++)
		{
			tmp.sprintf("aoDAC%dstate=%d\n",i+1,aoDACstate[i]) ;
			configfile->writeBlock ((const char *)tmp, tmp.length());
		}
		for (i=0;i<MaxDAchannel;i++)
		{
			tmp.sprintf("aoDA%dstate=%d\n",i+1,aoDAstate[i]) ;
			configfile->writeBlock ((const char *)tmp, tmp.length());
		}
		tmp.sprintf("\n#Digital IO states and types\n");
		configfile->writeBlock ((const char *)tmp,tmp.length());
		/* write io config */
		for (i=0;i<MaxIOchannel;i++)
		{
			tmp.sprintf("io%dstate=%d\nio%dtype=%d\n",i+1,iostate[i],i+1,iotype[i]);
			configfile->writeBlock ((const char *)tmp, tmp.length());
		}
		configfile->close();
	}
	else
		QMessageBox::message("Error","Error saving settings !");
}

void Settings::RestoreSettings()
{
	/* (re)read the file */
	int pos1;
	int ionumber;
	QString tmp;

	/* look for a config file */
	QString configfilename;
	configfilename = QString(getenv("HOME")) + QString("/.k8ktest");

	configfile = new QFile(configfilename);
	if ( configfile->open(IO_ReadOnly) )
	{
		QTextStream t( configfile );
		QString s;
		while ( !t.eof() )
		{
			s = t.readLine();
			/* trim the string */
			
			/* if '#' at first char, ignore  */
			if (s.left(1)!=QString("#"))
			{
				pos1 = s.find('=',0,FALSE);
				if(pos1!=-1)
				{
					if(s.mid(0,2)==QString("io")) //io config
					{
						if(s.mid(pos1-5,5)==QString("state"))
						{
							tmp = s.mid(2,pos1-7);
							ionumber = tmp.toInt();
							tmp = s.mid(pos1+1,s.length()-pos1+1);
							iostate[ionumber-1] = tmp.toInt();
						}
						
						else if(s.mid(pos1-4,4)==QString("type"))
						{
							tmp = s.mid(2,pos1-6);
							ionumber = tmp.toInt();
							tmp = s.mid(pos1+1,s.length()-pos1+1);
							iotype[ionumber-1] = tmp.toInt();
						}
					}
					if(s.mid(0,4)==QString("aoDA")) //ao config
					{
						if(s.mid(2,3)==QString("DAC"))
						{
							tmp = s.mid(5,pos1-10);
							ionumber = tmp.toInt();
							tmp = s.mid(pos1+1,s.length()-pos1+1);
							if(s.mid(pos1-5,5)==QString("state"))
							{
								tmp = s.mid(pos1+1,s.length()-pos1+1);
								aoDACstate[ionumber-1] = tmp.toInt();
							}
						}
						else
						{
							tmp = s.mid(4,pos1-9);
							ionumber = tmp.toInt();
							tmp = s.mid(pos1+1,s.length()-pos1+1);
							if(s.mid(pos1-5,5)==QString("state"))
							{
								tmp = s.mid(pos1+1,s.length()-pos1+1);
								aoDAstate[ionumber-1] = tmp.toInt();
							}
						}
					}
					if(s.mid(0,pos1)==QString("refreshrate"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);	refreshrate = tmp.toInt();}
					
					if(s.mid(0,pos1)==QString("i2cbusdelay"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);i2cbusdelay = tmp.toInt();}
					
					if(s.mid(0,pos1)==QString("chain"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);currentchain = tmp.toInt();}
					
					if(s.mid(0,pos1)==QString("card"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);currentcard = tmp.toInt();}
					
					if(s.mid(0,pos1)==QString("saveonexit"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);saveonexit = tmp.toInt();}

					if(s.mid(0,pos1)==QString("font"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);font = tmp;}

					if(s.mid(0,pos1)==QString("lptport"))
					{tmp = s.mid(pos1+1,s.length()-pos1+1);lptport = tmp.toInt();}
				}
			}
		}
		configfile->close();
	}
}

